#!/usr/bin/perl -w
use strict;

sub read_file {
  my($filename)=shift;
  my($line);
  my(@host_list);
  open(INPUT,"$filename") || die "Impossible to open platform file \"$filename\"\n";
  while(<INPUT>) {
    $line=$_;
    while($line !~ />\s*$/ and $line !~ /^\s*$/){
      chomp($line);
      $line = $line.<INPUT>;
    }
    if($line=~/host id="(\S*)"/) {
      push @host_list,$1;
    }
    if($line=~/cluster id/) {
      my $prefix = "";
      my	$rad_start = "";
      my	$rad_end = "";
      my	$suffix = "";
      if ($line =~ /prefix="([^"]*)"/){
        $prefix=$1;
    }
    if ($line =~ /suffix="([^"]*)"/){
      $suffix=$1;
    }
    if ($line =~ /radical="(\d*)-(\d*)"/){
      $rad_start=$1;
      $rad_end=$2;
    }
    foreach ($rad_start..$rad_end){
      my $host = $prefix.$_.$suffix;
      push @host_list,$host;
    }
    }
  }
  close(INPUT);
  return \@host_list;
}

sub generate_random_deployment{
  my($host_list,$filename)=@_;
  my($nhost) = scalar(@$host_list);

  open(CHUBS,"> $filename") || die "Impossible to open output file \"$filename\"\n";

  print CHUBS "<?xml version='1.0'?>\n";
  print CHUBS "<!DOCTYPE platform SYSTEM \"simgrid.dtd\">\n";
  print CHUBS "<platform version=\"2\">\n\n";   
  my $p;
  my $nb = $#$host_list + 1;
  foreach $p (@$host_list) {
    print CHUBS "  <process host=\"$p\" function='rendez_vous'>\n";
    print CHUBS "    <argument value=\"$nb\"/>\n";
    print CHUBS "  </process>\n\n";
  }
  print CHUBS "</platform>\n";

  close(CHUBS);
}

sub main {
  my($nodes,$edges,$interferences,$host_list,$count_interferences);

  scalar @ARGV >= 1 or die "Usage:\n\t./generate_random_deployment.pl <platform_file>\n\n";
  my($filename)=$ARGV[0];

  my($thisdir, @file_with_path);
  $thisdir=`pwd`;
  chomp $thisdir;
  my $dir_file=$filename;


# Cut off file path
  @file_with_path = split("/",$filename);
  $filename = $file_with_path[scalar(@file_with_path)-1];
  $dir_file=~s/\/[^\/]+$//g;
  $thisdir = $thisdir."/".$dir_file."/";

  $filename =~ s/\.xml$//g;
  $filename =~ s/-p$//g;
  $filename = $thisdir.$filename."-d.xml";
  print "$filename\n";

  $host_list = read_file $ARGV[0];
  generate_random_deployment($host_list,$filename);
}

main;
