/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include<stdio.h>

#include "msg/msg.h"            /* Yeah! If you want to use msg, you need to include msg/msg.h */
#include "xbt/sysdep.h"         /* calloc */

/* Create a log channel to have nice outputs. */
#include "xbt/log.h"
#include "xbt/asserts.h"

XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
                             "Messages specific for this msg example");

int sender(int argc, char *argv[]);
int receiver(int argc, char *argv[]);

MSG_error_t test_all(const char *platform_file,
                     const char *application_file);

typedef enum {
  PORT_22 = 0,
  MAX_CHANNEL
} channel_t;

/** Emitter function  */
int sender(int argc, char *argv[])
{
  m_host_t host = NULL;
  m_task_t task = NULL;
  char sprintf_buffer[64];

  INFO0("sender");

  /*host = xbt_new0(m_host_t,1); */

  INFO1("host = %s", argv[1]);

  host = MSG_get_host_by_name(argv[1]);

  if (host == NULL) {
    INFO1("Unknown host %s. Stopping Now! ", argv[1]);
    abort();
  }

  /* Send a task */
  sprintf(sprintf_buffer, "task");
  /* MSG_task create(name, computation size, communication size,
   * user_value);*/
  task = MSG_task_create(sprintf_buffer, 0.0, 10^4, NULL);
  sprintf(sprintf_buffer, "%s_%d", argv[1], PORT_22);
  MSG_task_send(task, sprintf_buffer);

  return 0;
}                               /* end_of_client */

/** Receiver function  */
int receiver(int argc, char *argv[])
{
  m_host_t host = MSG_host_self ();
  m_task_t task = NULL;
  char sprintf_buffer[64];
  int res;

  INFO0("receiver");

  sprintf(sprintf_buffer, "%s_%d", MSG_host_get_name(host), PORT_22);

  /* Receive a task */
  res = MSG_task_receive(&task, sprintf_buffer);
  if (res == MSG_OK) {
    INFO1("Task received : %s", task->name);
    MSG_task_destroy(task);
  } else {
    xbt_assert0(0, "Unexpected behavior");
  }

  return 0;
}                               /* end_of_receiver */


/** Test function */
MSG_error_t test_all(const char *platform_file,
                     const char *application_file)
{

  MSG_error_t res = MSG_OK;



  INFO0("test_all");

  /*  Simulation setting */
  MSG_set_channel_number(MAX_CHANNEL);
  MSG_create_environment(platform_file);

  /*   Application deployment */
  MSG_function_register("sender", sender);
  MSG_function_register("receiver", receiver);

  MSG_launch_application(application_file);

  res = MSG_main();

  return res;
}                               /* end_of_test_all */


/** Main function */
int main(int argc, char *argv[])
{
  MSG_error_t res = MSG_OK;


  MSG_global_init(&argc, argv);


  if (argc != 3) {
    CRITICAL1("Usage: %s platform_file deployment_file\n",
              argv[0]);
    CRITICAL1
        ("example: %s msg_platform.xml msg_deployment.xml\n",
         argv[0]);
    exit(1);
  }

  res = test_all(argv[1], argv[2]);

  INFO1("Total simulation time: %le", MSG_get_clock());

  MSG_clean();

  if (res == MSG_OK)
    return 0;
  else
    return 1;
}                               /* end_of_main */
